-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftLevelText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Level Text"] = true,
	["Level text options"] = true,
	["Enable"] = true,
	["Enable level text display on the nameplate"] = true,

	["Show Creature Type"] = true,	
	["Shows B,D,K,E,H,U for Beast, Demon, Dragonkin, Elemental, Humanoid and Undead creatures"] = true,

	["Show Classification"] = true,
	["Shows +,r,b for elite, rare and bosses after the level"] = true,
	["Show Race"] = true,
	["Shows unit race after the level"] = true,

	["Typeface"] = true,
	["Level text typeface options"] = true,
	["Font"] = true,
	["Sets the font for level text"] = true,
	["Font Size"] = true,
	["Sets the font height of the level text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on level text"] = true,
	["Outline"] = true,
	["Sets the font outline of the level text"] = true,

	["Position"] = true,
	["Adjust level text position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the level text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the level text"] = true,
	["X Offset"] = true,
	["X offset of the level text"] = true,
	["Y Offset"] = true,
	["Y offset of the level text"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Override Text"] = true,
	["Override level text"] = true,
	["Format"] = true,
	["Level tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local LEVELTEXT_FORMAT_BASE = "[level]"
local LEVELTEXT_FORMAT_CREATURETYPE = "[shortCreatureType]"
local LEVELTEXT_FORMAT_CLASSIFICATION = "[level:shortClassification][not(level):classification]"
local LEVELTEXT_FORMAT_RACE = "[race:prefix(\" \"):cyan]"

local AloftLevelText = Aloft:NewModule("LevelText", "AceEvent-2.0")
AloftLevelText.db = Aloft:AcquireDBNamespace("levelText")
Aloft:RegisterDefaults("levelText", "profile", {
	enable				= true,
	font				= "Arial Narrow",
	fontSize			= 8,
	outline				= "OUTLINE",
	point				= "RIGHT",
	relativeToPoint		= "RIGHT",
	offsetX				= 0,
	offsetY				= 0,
	showCreatureType	= false,
	showClassification	= false,
	showRace			= false,
	overrideText		= false,
	format				= "",
})

Aloft.Options.args.levelText =
{
	type = 'group',
	name = L["Level Text"],
	desc = L["Level text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable = 
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable level text display on the nameplate"],
			get = function() return AloftLevelText.db.profile.enable end,
			set = function(v) AloftLevelText.db.profile.enable = v AloftLevelText:UpdateTag() Aloft:DetermineDataSources() AloftLevelText:UpdateAll() end,
			order = 1
		},
		showCreatureType = 
		{
			type = 'toggle',
			name = L["Show Creature Type"],
			desc = L["Shows B,D,K,E,H,U for Beast, Demon, Dragonkin, Elemental, Humanoid and Undead creatures"],
			get = function() return AloftLevelText.db.profile.showCreatureType end,
			set = function(v) 
					AloftLevelText.db.profile.showCreatureType = v 
					AloftLevelText:UpdateTag()
					Aloft:DetermineDataSources()
					AloftLevelText:UpdateAll() 
				  end,
			order = 2,
		},
		showClassification = 
		{
			type = 'toggle',
			name = L["Show Classification"],
			desc = L["Shows +,r,b for elite, rare and bosses after the level"],
			get = function() return AloftLevelText.db.profile.showClassification end,
			set = function(v) 
					AloftLevelText.db.profile.showClassification = v 
					AloftLevelText:UpdateTag()
					Aloft:DetermineDataSources()
					AloftLevelText:UpdateAll() 
				  end,
			order = 3,
		},
		showRace = 
		{
			type = 'toggle',
			name = L["Show Race"],
			desc = L["Shows unit race after the level"],
			get = function() return AloftLevelText.db.profile.showRace end,
			set = function(v) 
					AloftLevelText.db.profile.showRace = v 
					AloftLevelText:UpdateTag()
					Aloft:DetermineDataSources()
					AloftLevelText:UpdateAll() 
				  end,
			order = 4,
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Level text typeface options"],
			disabled = function() return not AloftLevelText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for level text"],
					get = function() return AloftLevelText.db.profile.font end,
					set = function(v) AloftLevelText.db.profile.font = v AloftLevelText:UpdateAll() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the level text"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftLevelText.db.profile.fontSize end,
					set = function(value) AloftLevelText.db.profile.fontSize = value AloftLevelText:UpdateAll() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on level text"],
					get = function() return AloftLevelText.db.profile.shadow end,
					set = function(v) AloftLevelText.db.profile.shadow = v AloftLevelText:UpdateAll() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font outline of the level text"],
					get = function() return AloftLevelText.db.profile.outline end,
					set = function(value) AloftLevelText.db.profile.outline = value AloftLevelText:UpdateAll() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust level text position"],
			disabled = function() return not AloftLevelText.db.profile.enable end,
			args =
			{
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the level text"],
					disabled = function() return not AloftLevelText.db.profile.enable end,
					get = function() return AloftLevelText.db.profile.point end,
					set = function(v) AloftLevelText.db.profile.point = v AloftLevelText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the level text"],
					disabled = function() return not AloftLevelText.db.profile.enable end,
					get = function() return AloftLevelText.db.profile.relativeToPoint end,
					set = function(v) AloftLevelText.db.profile.relativeToPoint = v AloftLevelText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the level text"],
					min = -32,
					max = 32,
					step = 1,
					disabled = function() return not AloftLevelText.db.profile.enable end,
					get = function() return AloftLevelText.db.profile.offsetX end,
					set = function(v) AloftLevelText.db.profile.offsetX = v AloftLevelText:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the level text"],
					min = -32,
					max = 32,
					step = 1,
					disabled = function() return not AloftLevelText.db.profile.enable end,
					get = function() return AloftLevelText.db.profile.offsetY end,
					set = function(v) AloftLevelText.db.profile.offsetY = v AloftLevelText:UpdateAll() end
				},
			},
		},
		advanced  =
		{
			type = 'group',
			name = L["Advanced"],
			desc = L["Options for expert users"],
			order = -1,
			disabled = function() return not AloftLevelText.db.profile.enable end,
			args =
			{
				enable = 
				{
					type = 'toggle',
					name = L["Override Text"],
					desc = L["Override level text"],
					get = function() return AloftLevelText.db.profile.overrideText end,
					set = function(v) AloftLevelText.db.profile.overrideText = v if v then AloftLevelText:UpdateTag() end Aloft:DetermineDataSources() AloftLevelText:UpdateAll() end,
					order = 1
				},
				format = 
				{
					type = 'text',
					name = L["Format"],
					desc = L["Level tag"],
					usage = L["<Any tag string>"],
					disabled = function() return not AloftLevelText.db.profile.overrideText end,
					get = function() return AloftLevelText.db.profile.format end,
					set = function(v) AloftLevelText.db.profile.format = v Aloft:DetermineDataSources() AloftLevelText:UpdateAll() end,
				}
			},
		},
	},
}

-----------------------------------------------------------------------------

local textMethod = nil
local textMethodData = { }

-----------------------------------------------------------------------------

function AloftLevelText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		local levelTextRegion = aloftData.levelTextRegion
	
		if self.db.profile.enable then
			if self.db.profile.overrideText then
				levelTextRegion:SetText(textMethod(aloftData))
				levelTextRegion:Show()
			else
				levelTextRegion:SetText(aloftData.level or -1)
				if aloftData.level then
					levelTextRegion:Show()
				else
					levelTextRegion:Hide()
				end
			end
		else
			levelTextRegion:Hide()
		end
		self:PrepareText(levelTextRegion, self.db.profile)
		self:PlaceFrame(levelTextRegion, aloftData.healthBar, self.db.profile)
	end
end

function AloftLevelText:UpdateTag()
	if not self.db.profile.enable or (not self.db.profile.showCreatureType and not self.db.profile.showClassification and not self.db.profile.showRace) then
		self.db.profile.overrideText = false
	else
		self.db.profile.overrideText = true
		self.db.profile.format = LEVELTEXT_FORMAT_BASE..
								  (self.db.profile.showCreatureType and LEVELTEXT_FORMAT_CREATURETYPE or "")..
								  (self.db.profile.showClassification and LEVELTEXT_FORMAT_CLASSIFICATION or "")..
								  (self.db.profile.showRace and LEVELTEXT_FORMAT_RACE or "")
	end
end

function AloftLevelText:RegisterEvents()
	local profile = self.db.profile

	self:UnregisterAllEvents()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:SetAll", function(type, value)
										if self.db.profile[type] then
											self.db.profile[type] = value
											self:UpdateAll()
										end
									  end)

	if self.db.profile.enable then
		if self.db.profile.overrideText then
			for events in pairs(textMethodData.events) do
				self:RegisterEvent(events, "UpdateText")
			end
			self:RegisterEvent("Aloft:OnNameplateShow", "PlaceAndUpdateLevelTextOnNameplateShow")
		else
			self:RegisterEvent("Aloft:OnNameplateShow", "PlaceLevelTextOnNameplateShow")
		end
	else
		self:RegisterEvent("Aloft:OnNameplateShow", "HideLevelTextOnNameplateShow")
	end
end

local dataRequiredList = { }
function AloftLevelText:RequiresData()
	local profile = self.db.profile

	textMethodData = AloftTag:CreateTag(profile.format)
	textMethod = textMethodData.method

	self:RegisterEvents()

	if not profile.enable or not profile.overrideText then
		return
	end

	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	for data in pairs(textMethodData.data) do
		table.insert(dataRequiredList, data)
	end

	return unpack(dataRequiredList)
end

-----------------------------------------------------------------------------

function AloftLevelText:OnEnable()
	self:UpdateAll()
end

function AloftLevelText:OnDisable()
	self:UnregisterAllEvents()

	for aloftData in Aloft:IterateNameplates() do
		local levelTextRegion = aloftData.levelTextRegion

		levelTextRegion:ClearAllPoints()
		levelTextRegion:SetPoint("CENTER", aloftData.nameplateFrame, "BOTTOMRIGHT", -13.331, 10.288)
		levelTextRegion:SetFont(NAMEPLATE_FONT, 11)		
		levelTextRegion:SetShadowOffset(1, -1)
		levelTextRegion:SetAlpha(1)
		levelTextRegion:SetText(aloftData.level or -1)
	end
end

function AloftLevelText:SetupFrame(aloftData)
	local levelTextRegion = aloftData.levelTextRegion

	self:PrepareText(levelTextRegion, self.db.profile)
	self:PlaceFrame(levelTextRegion, aloftData.healthBar, self.db.profile)
end

function AloftLevelText:HideLevelTextOnNameplateShow(aloftData)
	aloftData.levelTextRegion:Hide()
end

function AloftLevelText:PlaceLevelTextOnNameplateShow(aloftData)
	self:PlaceFrame(aloftData.levelTextRegion, aloftData.healthBar, self.db.profile)
end

function AloftLevelText:UpdateText(aloftData)
	aloftData.levelTextRegion:SetText( textMethod(aloftData) )
end

function AloftLevelText:PlaceAndUpdateLevelTextOnNameplateShow(aloftData)
	local levelTextRegion = aloftData.levelTextRegion
	if aloftData.isBoss then
		levelTextRegion:SetTextColor(1, 0, 0)
	end
	self:PlaceFrame(levelTextRegion, aloftData.healthBar, self.db.profile)
	levelTextRegion:Show()
	self:UpdateText(aloftData)
end

-----------------------------------------------------------------------------
