-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBarTimeText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Time Text"] = true,
	["Cast bar time text options"] = true,
	["Enable"] = true,
	["Show cast bar time text"] = true,
	["Typeface"] = true,
	["Cast bar time text typeface options"] = true,
	["Font"] = true, 
	["Sets the font for cast bar time text"] = true,
	["Font Size"] = true,
	["Sets the font height of the cast bar time text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on cast bar time text"] = true,
	["Alignment"] = true,
	["Sets the alignment of the cast bar time text"] = true,
	["Outline"] = true,
	["Sets the font outline for cast bar time text"] = true,
	["Position"] = true,
	["Adjust cast bar time text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	["Color"] = true,
	["Sets the cast bar time text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local castBarTimeTextRegion = nil

-----------------------------------------------------------------------------

local AloftCastBarTimeText = Aloft:NewModule("CastBarTimeText", "AceEvent-2.0")
AloftCastBarTimeText.db = Aloft:AcquireDBNamespace("castBarTimeText")
Aloft:RegisterDefaults("castBarTimeText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= true,
	alignment		= "RIGHT",
	outline			= "",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	color			= { 1, 1, 1, 1 }
})

Aloft.Options.args.castBar.args.castBarTimeText =
{
	type = 'group',
	name = L["Time Text"],
	desc = L["Cast bar time text options"],
	disabled = function() return not Aloft:IsActive() or GetCVar("ShowVKeyCastbar") ~= "1" end,
	order = -1,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.enable end,
			set = function(v) AloftCastBarTimeText.db.profile.enable = v AloftCastBarTimeText:Update() end,
			order = 1
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Cast bar time text typeface options"],
			disabled = function() return not AloftCastBarTimeText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for cast bar time text"],
					get = function() return AloftCastBarTimeText.db.profile.font end,
					set = function(v) AloftCastBarTimeText.db.profile.font = v AloftCastBarTimeText:Update() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the cast bar time text"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.fontSize end,
					set = function(value) AloftCastBarTimeText.db.profile.fontSize = value AloftCastBarTimeText:Update() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on cast bar time text"],
					get = function() return AloftCastBarTimeText.db.profile.shadow end,
					set = function(v) AloftCastBarTimeText.db.profile.shadow = v AloftCastBarTimeText:Update() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font outline for cast bar time text"],
					get = function() return AloftCastBarTimeText.db.profile.outline end,
					set = function(value) AloftCastBarTimeText.db.profile.outline = value AloftCastBarTimeText:Update() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the cast bar time text"],
			disabled = function() return not AloftCastBarTimeText.db.profile.enable end,
			get = function() return AloftCastBarTimeText.db.profile.alignment end,
			set = function(value) AloftCastBarTimeText.db.profile.alignment = value AloftCastBarTimeText:Update() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust cast bar time text position"],
			disabled = function() return not AloftCastBarTimeText.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.left end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.left = v AloftCastBarTimeText:Update() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.right end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.right = v AloftCastBarTimeText:Update() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.vertical end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.vertical = v AloftCastBarTimeText:Update() end
				},
			},
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the cast bar time text color"],
			disabled = function() return not AloftCastBarTimeText.db.profile.enable end,
			get = function() return unpack(AloftCastBarTimeText.db.profile.color) end,
			set = function(r, g, b, a) AloftCastBarTimeText.db.profile.color = { r, g, b, a } AloftCastBarTimeText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBarTimeText:Update()
	self:CreateFrame()
	self:RegisterEvents()
	
	if self.db.profile.enable then
		for aloftData in Aloft:IterateVisibleNameplates() do
			self:SetupFrame(aloftData)
		end
	end
end

-----------------------------------------------------------------------------

function AloftCastBarTimeText:CreateFrame()
	if not self.db.profile.enable then
		if castBarTimeTextRegion then
			castBarTimeTextRegion:Hide()
		end
		return
	end

	if not castBarTimeTextRegion then
		castBarTimeTextRegion = UIParent:CreateFontString()
	end
	
	self:PrepareText(castBarTimeTextRegion, self.db.profile)
end

function AloftCastBarTimeText:RegisterEvents()
	if self.db.profile.enable then	
		self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
		self:RegisterEvent("Aloft:OnCastBarValueChanged", "UpdateText")
		self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
		self:RegisterEvent("Aloft:OnCastBarHide", "OnCastBarHide")
	else
		self:CancelEvent("Aloft:SetupFrame")
		self:CancelEvent("Aloft:OnCastBarValueChanged")
		self:CancelEvent("Aloft:OnCastBarShow")
		self:CancelEvent("Aloft:OnCastBarHide")
	end
end

-----------------------------------------------------------------------------

function AloftCastBarTimeText:OnEnable()	
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:Update()
		end
	end)
	
	self:Update()
end

function AloftCastBarTimeText:OnDisable()
	self:UnregisterAllEvents()
	
	if castBarTimeTextRegion then
		castBarTimeTextRegion:Hide()
	end
end

function AloftCastBarTimeText:SetupFrame(aloftData)
	aloftData.lastCastTimeValue = 0
	if aloftData:IsCurrentTarget() then
		self:OnCastBarShow(aloftData)
	end	
end

function AloftCastBarTimeText:UpdateText(aloftData)
	local castBar = aloftData.castBar
	local minValue, maxValue = castBar:GetMinMaxValues()
	local value = castBar:GetValue()

	if value >= maxValue or value <= minValue then
		castBarTimeTextRegion:SetText("")
		aloftData.lastCastTimeValue = 0
	else
		if value > aloftData.lastCastTimeValue then
			castBarTimeTextRegion:SetText(string.format("%.2f", maxValue-value))
			aloftData.lastCastTimeValue = value
		else
			castBarTimeTextRegion:SetText(string.format("%.2f", value-minValue))
			aloftData.lastCastTimeValue = value
		end
	end
end

function AloftCastBarTimeText:OnCastBarShow(aloftData)
	local castBar = aloftData.castBar
	local profile = self.db.profile

	aloftData.lastCastTimeValue = castBar:GetValue()
	castBarTimeTextRegion:ClearAllPoints()
	castBarTimeTextRegion:SetPoint("TOPLEFT", castBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
	castBarTimeTextRegion:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
	castBarTimeTextRegion:SetParent(aloftData.nameplateFrame)
	castBarTimeTextRegion:Show()
end

function AloftCastBarTimeText:OnCastBarHide(aloftData)
	castBarTimeTextRegion:Hide()
end

-----------------------------------------------------------------------------
