-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCommentData")
local gratuity = AceLibrary("Gratuity-2.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Comments"] = true,
	["Comment options"] = true,

	["Save Comment Data"] = true,
	["Stores comment data between sessions"] = true,

	["Automatic NPC Comments"] = true,
	["Automatically set NPC comments from their tags"] = true,

    ["Reset Comments"] = true,
	["Empties the comment database"] = true,
	
	["Level"] = true,
} end)

-----------------------------------------------------------------------------

local comments = { }

-----------------------------------------------------------------------------

local AloftCommentData = Aloft:NewModule("CommentData", "AceEvent-2.0")
AloftCommentData.db = Aloft:AcquireDBNamespace("comment")
Aloft:RegisterDefaults("comment", "account", { comments = { } })
Aloft:RegisterDefaults("comment", "profile", { save = true, autoNPC = true })

-----------------------------------------------------------------------------

Aloft.Options.args.data.args.comment =
{
	type = 'group',
	name = L["Comments"],
	desc = L["Comment options"],
	args =
	{
		save =
		{
			type = 'toggle',
			name = L["Save Comment Data"],
			desc = L["Stores comment data between sessions"],
			get = function() return AloftCommentData.db.profile.save end,
			set = function(v)
					AloftCommentData.db.profile.save = v
					if v then
						AloftCommentData.db.server.comments = comments
					else
						AloftCommentData.db.server.comments = comments
					end
				  end,
		},
		autoNPC =
		{
			type = 'toggle',
			name = L["Automatic NPC Comments"],
			desc = L["Automatically set NPC comments from their tags"],
			get = function() return AloftCommentData.db.profile.autoNPC end,
			set = function(v)
					AloftCommentData.db.profile.autoNPC = v
					AloftCommentData:RegisterEvents()
				  end,
		},
		clear =
		{
			type = "execute",
			name = L["Reset Comments"],
			desc = L["Empties the comment database"],
			func = function() AloftCommentData:ClearAllData() end,
		},
	}
}

-----------------------------------------------------------------------------

AloftTagDirectAccessData.comment = true
AloftTagData.comment = { data = "comment", events = "Aloft:OnCommentDataChanged" }

function AloftCommentData:ProvidesData()
	return "comment"
end

function AloftCommentData:RegisterEvents()
	self:UnregisterAllEvents()
	if self.db.profile.autoNPC then
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdateCommentFromMouseOver")
		self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateCommentFromTarget")
	end
end

function AloftCommentData:EnableDataSource()
	self:RegisterEvents()
end

-----------------------------------------------------------------------------

function AloftCommentData:OnEnable()
	if self.db.profile.save then
		comments = self.db.account.comments
	end
end

function AloftCommentData:UpdateData(aloftData)
	aloftData.comment = comments[aloftData.name]
end

-----------------------------------------------------------------------------

function AloftCommentData:UpdateAll()
	for aloftData in Aloft:IterateVisibleNameplates() do
		local comment = comments[aloftData.name]
		if aloftData.comment ~= comment then
			aloftData.comment = comment
			self:TriggerEvent("Aloft:OnCommentDataChanged", aloftData)
		end
	end
end

function AloftCommentData:SetCommentForTarget(comment)
	if UnitExists("target") then
		local name = UnitName("target")
		comments[name] = comment ~= "" and comment or nil
		self:UpdateAll()
		return name
	end
end

function AloftCommentData:ClearAllData()
	for k in pairs(comments) do
		comments[k] = nil
	end
	self:UpdateAll()
end

-----------------------------------------------------------------------------

function AloftCommentData:UpdateCommentFromUnit(unitid)
	if UnitExists(unitid) and not UnitPlayerControlled(unitid) then
		local key = UnitName(unitid)
		
		if not comments[key] then
			gratuity:SetUnit(unitid)
			local comment = gratuity:GetLine(2)

			if comment and comment ~= "" and comment:find(L["Level"]) ~= 1 then
				comments[key] = comment

				for aloftData in Aloft:IterateVisibleNameplates() do
					if aloftData.name == key then
						aloftData.comment = comment
						self:TriggerEvent("Aloft:OnCommentDataChanged", aloftData)
					end
				end
			end

		end
	end
end

function AloftCommentData:UpdateCommentFromMouseOver()
	self:UpdateCommentFromUnit("mouseover")
end

function AloftCommentData:UpdateCommentFromTarget()
	self:UpdateCommentFromUnit("target")
end

-----------------------------------------------------------------------------
